unit ICMappings;

interface

	uses
{$ifc undefined THINK_Pascal}
		Types, Files, Aliases, Errors,
{$endc}
		ICTypes, ICAPI, ICKeys;

	function ICCountEntries (entries: Handle; var count: longint): ICError;
	function ICMGetEntry (entries: handle; pos: longInt; var entry: ICMapEntry): ICError;
	function ICMGetIndEntry (entries: handle; ndx: longint; var pos: longint; var entry: ICMapEntry): ICError;
	function ICMAddEntry (entries: handle; var entry: ICMapEntry): ICError;
	function ICMDeleteEntry (entries: handle; pos: longint): ICError;
	function ICMSetEntry (entries: handle; pos: longInt; var entry: ICMapEntry): ICError;

implementation

{$ifc undefined THINK_Pascal}
	uses
		Memory, ToolUtils;
{$endc}

	procedure UnpackEntry (p: ptr; var entry: ICMapEntry);
(* WARNING: Depends very much on the exact format of ICMapEntry! *)
		procedure CopyString (var p: ptr; var s: str255);
			var
				len: integer;
		begin
			len := BAND(p^, $FF) + 1;
			BlockMove(p, @s, len);
			p := ptr(ord(p) + len);
		end;
	begin
		BlockMove(p, @entry, 6);
		BlockMove(p, @entry, entry.fixed_length);
		p := ptr(ord(p) + entry.fixed_length);
		CopyString(p, entry.extension);
		CopyString(p, entry.creator_app_name);
		CopyString(p, entry.post_app_name);
		CopyString(p, entry.MIME_type);
		CopyString(p, entry.entry_name);
	end;

	procedure PackEntry (var entry: ICMapEntry; p: ptr);
		procedure CopyString (var s: str255);
		begin
			BlockMove(@s, ptr(ord(p) + entry.total_length), length(s) + 1);
			entry.total_length := entry.total_length + length(s) + 1;
		end;
	begin
		entry.version := 0;
		entry.fixed_length := ord(@entry.extension) - ord(@entry);
		entry.total_length := entry.fixed_length;
		CopyString(entry.extension);
		CopyString(entry.creator_app_name);
		CopyString(entry.post_app_name);
		CopyString(entry.MIME_type);
		CopyString(entry.entry_name);
		entry.total_length := entry.total_length + ord(odd(entry.total_length));
		BlockMove(@entry, p, entry.fixed_length);
	end;

	function ICMDeleteEntry (entries: handle; pos: longint): ICError;
		var
			entry: ICMapEntry;
			lerr: longint;
	begin
		UnpackEntry(ptr(ord(entries^) + pos), entry);
		lerr := Munger(entries, pos, nil, entry.total_length, Ptr(-1), 0);
		if lerr >= 0 then begin
			lerr := noErr;
		end; (* if *)
		ICMDeleteEntry := lerr;
	end; (* ICMDeleteEntry *)

	function ICCountEntries (entries: Handle; var count: longint): ICError;
		var
			p: integerPtr;
			pos: longint;
	begin
		p := integerPtr(entries^);
		pos := 0;
		count := 0;
		while pos < GetHandleSize(entries) do begin
			pos := pos + p^;
			p := integerPtr(ord(p) + p^);
			count := count + 1;
		end; (* while *)
		ICCountEntries := noErr;
	end; (* ICCountEntries *)

	function ICMGetEntry (entries: handle; pos: longInt; var entry: ICMapEntry): ICError;
		var
			err: OSErr;
	begin
		if (0 <= pos) & (pos < GetHandleSize(entries)) then begin
			UnpackEntry(ptr(ord(entries^) + pos), entry);
			err := noErr;
		end
		else begin
			err := paramErr;
		end;
		ICMGetEntry := err;
	end;

	function ICMGetIndEntry (entries: handle; ndx: longint; var pos: longint; var entry: ICMapEntry): ICError;
		var
			err: ICError;
			p: integerPtr;
			i: longint;
	begin
		p := integerPtr(entries^);
		pos := 0;
		while (ndx > 1) & (pos < GetHandleSize(entries)) do begin
			pos := pos + p^;
			p := integerPtr(ord(p) + p^);
			ndx := ndx - 1;
		end; (* while *)
		ICMGetIndEntry := ICMGetEntry(entries, pos, entry);
	end; (* ICMGetIndEntry *)

	function ICMAddEntry (entries: handle; var entry: ICMapEntry): ICError;
		var
			e: ICMapEntry;
	begin
		PackEntry(entry, @e);
		ICMAddEntry := PtrAndHand(@e, entries, entry.total_length);
	end;

	function ICMSetEntry (entries: handle; pos: longInt; var entry: ICMapEntry): ICError;
		var
			err: ICError;
			e: ICMapEntry;
			oldlen: integer;
	begin
		if (0 <= pos) & (pos < GetHandleSize(entries)) then begin
			BlockMove(ptr(ord(entries^) + pos), @e, 6);
			oldlen := e.total_length;
			PackEntry(entry, @e);
			err := Munger(entries, pos, nil, oldlen, @e, entry.total_length);
			if err >= 0 then begin
				err := noErr;
			end;
		end
		else begin
			err := paramErr;
		end;
		ICMSetEntry := err;
	end;

end. (* ICMappings *)